#ifndef _NSTRUCT_HEADER_H
#define _NSTRUCT_HEADER_H

#ifdef	__cplusplus
extern "C"
{
#endif

#define longword	unsigned long

#define longbool	unsigned long

// coordinates:
typedef double		xyz [3];

// node = end of rod where links are connected:
typedef struct
{
	xyz p;
	longbool fix; // fixed or free node
} node [2];

// parameters of one rod:
typedef	struct
{
	node s;
	double	lRod;
} TRod;
typedef	TRod *TabTRod; // all rods

// parameters of one link:
typedef	struct
{
	// link between rod1 node1 and rod2 node2:
	longword	r1, n1,	r2, n2;
	// # of independant module:
	longword	nMod;
	// length of unstretched link:
	double		lLnk;
	// 1/2 * stiffness of link:
	double		k05;
} TLnk;
typedef	TLnk *TabTLnk; // all links

// coordinates of one external force vector:
typedef xyz TForce [2];
typedef TForce *TabTForce; // all external applied forces

// if Lagrange multipliers are to be reused in successive calculations:
typedef double *TabDouble;

// the structure to be calculated:
typedef struct
{
	longword		nb, nl;	// # of rods and of links
	TabTRod			rd;		// system of rods
	TabTLnk			lk;		// links between rods
	TabTForce		fc;		// external forces applied to the system (optional)
	TabDouble		lg;		// user's Lagrange multipliers (optional), MUST be initialized at 0.0 if used
	void			*fe;	// customed elastic forces (optional)
	void			*ce;	// external constraints (optional)
	void			*p1, *p2; // parameters for fe and ce (optional)
} TSyst;
typedef TSyst *pTSyst;

// user defined:
typedef double (__stdcall *TExtern) (pTSyst syst);
typedef TExtern *TabTExtern; 

// callback:
typedef void (__stdcall *TCallback) (pTSyst syst, longword *opt);

#ifndef	IN
	#define	IN
#endif

#ifndef	OUT
	#define	OUT
#endif

//########################################################################################//
//######################### F U N C T I O N S  O F  T H E  D L L #########################//
//########################################################################################//

// Solve: solves the system of rods and links
// IN: sys = structure to be calculated
// IN: cbk = address of callback procedure (optional)
// OUT: sys = calculated structure
// Return: result of calculation (see NStruct_MsgCodes)
typedef longword (__stdcall *TSolve) (pTSyst syst,
									  TCallback cbk,
									  longword *opt);
TSolve Solve;

// GetHalfStiffnessValue: retrieves default half stiffness value
// IN: none
// OUT: none
// Return: default half stiffness value
typedef double (__stdcall *TGetHalfStiffnessValue) ();
TGetHalfStiffnessValue GetHalfStiffnessValue;

// SetHalfStiffnessValue: changes default half stiffness value
// IN: new default half stiffness value
// OUT: none
// Return: none
// typedef void (__stdcall *TSetHalfStiffnessValue) (double);
// TSetHalfStiffnessValue SetHalfStiffnessValue; NOT IN DEMO VERSION

// NukeThemAll: kill all internal loops and threads
// IN: none
// OUT: none
// Return: none
typedef void (__stdcall *TNukeThemAll) ();
TNukeThemAll NukeThemAll;

// Distance: distance between two points
// IN: coordinates of points
// OUT: none
// Return: calculated distance
typedef double (__stdcall *TDistance) (xyz *, xyz *);
TDistance Distance;

// Modules: # of independant modules of the system
// IN: structure to be calculated
// OUT: structure to be calculated (links are "tagged")
// Return: # of modules
typedef longword (__stdcall *TModules) (pTSyst syst, longbool opt);
TModules Modules;

// Potential: potential energy of the system
// IN: structure to be calculated
// OUT: none
// Return: potential energy
typedef double (__stdcall *TPotential) (pTSyst syst);
TPotential Potential;

#ifdef	__cplusplus
}
#endif

#endif // _NSTRUCT_HEADER_H