#include <stdio.h>
#include <windows.h>
#include "NStruct_Header.h"
#include "NStruct_MsgCodes.h"

longword Compt = 0;
longword Opts = 0xBFB60DB; // see "Bits option map.txt"
longword Res;
TSyst Syst;
HINSTANCE Handle;

/* importation of the dll functions */
longword LoadFunctionsFromLibrary (HINSTANCE *hdl)
{
	*hdl = LoadLibrary ("Structure.dll");
	if (*hdl != NULL)
	{
		Solve = (TSolve) GetProcAddress (*hdl, "Solve");
		GetHalfStiffnessValue = (TGetHalfStiffnessValue) GetProcAddress (*hdl, "GetHalfStiffnessValue"); 
		// SetHalfStiffnessValue = (TSetHalfStiffnessValue) GetProcAddress (*hdl, "SetHalfStiffnessValue"); NOT IN DEMO VERSION
		NukeThemAll = (TNukeThemAll) GetProcAddress (*hdl, "NukeThemAll");
		Distance = (TDistance) GetProcAddress (*hdl, "Distance"); 
		Modules = (TModules) GetProcAddress (*hdl, "Modules"); 
		Potential = (TPotential) GetProcAddress (*hdl, "Potential");
		if (!Solve || !GetHalfStiffnessValue || /* !SetHalfStiffnessValue || */
			!NukeThemAll || !Distance || !Modules || !Potential) return STRUCT_ERR;
	}
	return STRUCT_OK;
}

/* initialization of the rods + cables system */
longword InitSyst (TSyst *sys, HINSTANCE *hdl)
{
	longword i;

	LoadFunctionsFromLibrary (hdl);
	if (!*hdl) return STRUCT_ERR;
	// 3 rods and 9 cables tensegrity structure:
	sys->nb = 3;
	sys->nl = 9;
	// initial guess:
	sys->rd = calloc (sys->nb, sizeof (TRod)); // memory allocation for rods' array
	sys->rd [0].s [0].p [1] = 0.0; sys->rd [0].s [0].p [2] = 0.0; sys->rd [0].s [0].p [0] = 0.0;
	sys->rd [0].s [1].p [1] = 1.0; sys->rd [0].s [1].p [2] = 0.0; sys->rd [0].s [1].p [0] = 0.0;
	sys->rd [0].s [0].fix = 1; sys->rd [0].s [1].fix = 1;
	sys->rd [0].lRod = 1.0; 
	sys->rd [1].s [0].p [1] = 0.0; sys->rd [1].s [0].p [2] = 0.0; sys->rd [1].s [0].p [0] = 0.0;
	sys->rd [1].s [1].p [1] = 0.0; sys->rd [1].s [1].p [2] = 1.0; sys->rd [1].s [1].p [0] = 0.0;
	sys->rd [1].s [0].fix = 0; sys->rd [1].s [1].fix = 0;
	sys->rd [1].lRod = 1.0; 
	sys->rd [2].s [0].p [1] = 0.0; sys->rd [2].s [0].p [2] = 0.0; sys->rd [2].s [0].p [0] = 0.0;
	sys->rd [2].s [1].p [1] = 0.0; sys->rd [2].s [1].p [2] = 0.0; sys->rd [2].s [1].p [0] = 1.0;
	sys->rd [2].s [0].fix = 0; sys->rd [2].s [1].fix = 0;
	sys->rd [2].lRod = 1.0; 
	// description of links between rods' ends:
	sys->lk = calloc (sys->nl, sizeof (TLnk)); // memory allocation for cables' array
	for (i = 0; i <= sys->nl - 1; i++)
	{
		sys->lk [i].lLnk = 0.1;
		sys->lk [i].k05 = 0.01;
	}
	sys->lk [0].r1 = 0; sys->lk [0].n1 = 1; sys->lk [0].r2 = 2;sys->lk [0].n2 = 1;
	sys->lk [1].r1 = 0; sys->lk [1].n1 = 0; sys->lk [1].r2 = 2;sys->lk [1].n2 = 1;
	sys->lk [2].r1 = 1; sys->lk [2].n1 = 0; sys->lk [2].r2 = 0;sys->lk [2].n2 = 1;
	sys->lk [3].r1 = 1; sys->lk [3].n1 = 0; sys->lk [3].r2 = 0;sys->lk [3].n2 = 0;
	sys->lk [4].r1 = 0; sys->lk [4].n1 = 0; sys->lk [4].r2 = 2;sys->lk [4].n2 = 0;
	sys->lk [5].r1 = 1; sys->lk [5].n1 = 1; sys->lk [5].r2 = 0;sys->lk [5].n2 = 1;
	sys->lk [6].r1 = 1; sys->lk [6].n1 = 0; sys->lk [6].r2 = 2;sys->lk [6].n2 = 0;
	sys->lk [7].r1 = 2; sys->lk [7].n1 = 1; sys->lk [7].r2 = 1;sys->lk [7].n2 = 1;
	sys->lk [8].r1 = 2; sys->lk [8].n1 = 0; sys->lk [8].r2 = 1;sys->lk [8].n2 = 1;
	sys->fc = NULL;
	sys->lg = NULL;
	sys->fe = NULL; // NULL in DEMO VERSION
	sys->ce = NULL; // NULL in DEMO VERSION
	sys->p1 = NULL; // NULL in DEMO VERSION
	sys->p2 = NULL; // NULL in DEMO VERSION

	return STRUCT_OK;
}

/* freeing the ressources */
void FreeSyst (TSyst sys, HINSTANCE hdl)
{
	FreeLibrary (hdl);
	free ((void *) sys.rd);
	free ((void *) sys.lk);
}

/* display of intermediate results */
void PrintRes (TSyst sys)
{
	longword i, j, k;

	for (i = 0; i <= sys.nb - 1; i++)
		for (j = 0; j <= 1; j++)
		{
			printf ("rod #%u\tend #%u\t", i + 1, j + 1);
			for (k = 0; k <= 2; k++) printf ("%.3f\t", sys.rd [i].s [j].p [k]);
			printf ("\n");
		}
	printf (">>> Potential= %.6e\n", Potential (&sys));
	getchar ();
}

/* allow interactivity with user */
void MyCallback (pTSyst sys, longword *opt)
{
	Compt++;
	printf ("iteration # %u\n", Compt);
	PrintRes (*sys);
}

/* test program */
void main (void)
{
	char msg [256];

	if (InitSyst (&Syst, &Handle) != STRUCT_OK) return;
	printf ("start: RETURN to calculate next iteration\n");
	PrintRes (Syst);
	Res = Solve ((TSyst *) &Syst, (TCallback) MyCallback, (longword *) &Opts);
	printf ("result: %s ; iteration # %u\n", GetStrError (Res, (char *) &msg), Compt);
	PrintRes (Syst);
	FreeSyst (Syst, Handle);

	return;
}