program StructPas;

uses NStruct_Header, NStruct_MsgCodes;

var Compt : longword = 0;
    Opts : longword = $BFB60DB; // see "Bits option map.txt"
    Syst : TSyst;
    Res : longword;

(* initialization of the rods + cables system *)
procedure InitSyst (var sys : TSyst);
var i : integer;
begin
 with sys do
  begin
   // 3 rods and 9 cables tensegrity structure:
   nb := 3;
   nl := 9;
   // initial guess:
   SetLength (rd, nb); // it's a dynamic array here, but a structure like
                       // TabTRod = array [0, 0] of TRod; pTabTRod = ^TabTRod;
                       // could be as well used ; in that case: GetMem (rd, nb * SizeOf (TRod))
                       // and FreeMem (rd) at the end of calculation
   rd [0].s [0].p [1] := 0.0; rd [0].s [0].p [2] := 0.0; rd [0].s [0].p [0] := 0.0;
   rd [0].s [1].p [1] := 1.0; rd [0].s [1].p [2] := 0.0; rd [0].s [1].p [0] := 0.0;
   rd [0].s [0].fix := true; rd [0].s [1].fix := true;
   rd [0].lRod := 1.0;
   rd [1].s [0].p [1] := 0.0; rd [1].s [0].p [2] := 0.0; rd [1].s [0].p [0] := 0.0;
   rd [1].s [1].p [1] := 0.0; rd [1].s [1].p [2] := 1.0; rd [1].s [1].p [0] := 0.0;
   rd [1].s [0].fix := false; rd [1].s [1].fix := false;
   rd [1].lRod := 1.0;
   rd [2].s [0].p [1] := 0.0; rd [2].s [0].p [2] := 0.0; rd [2].s [0].p [0] := 0.0;
   rd [2].s [1].p [1] := 0.0; rd [2].s [1].p [2] := 0.0; rd [2].s [1].p [0] := 1.0;
   rd [2].s [0].fix := false; rd [2].s [1].fix := false;
   rd [2].lRod := 1.0;
   // description of links between rods' ends:
   SetLength (lk, nl); // see commentary for rd above
   for i := 0 to nl - 1 do
    begin
     lk [i].lLnk := 0.1;
     lk [i].k05 := 0.01;
    end;
   lk [0].r1 := 0; lk [0].n1 := 1; lk [0].r2 := 2; lk [0].n2 := 1;
   lk [1].r1 := 0; lk [1].n1 := 0; lk [1].r2 := 2; lk [1].n2 := 1;
   lk [2].r1 := 1; lk [2].n1 := 0; lk [2].r2 := 0; lk [2].n2 := 1;
   lk [3].r1 := 1; lk [3].n1 := 0; lk [3].r2 := 0; lk [3].n2 := 0;
   lk [4].r1 := 0; lk [4].n1 := 0; lk [4].r2 := 2; lk [4].n2 := 0;
   lk [5].r1 := 1; lk [5].n1 := 1; lk [5].r2 := 0; lk [5].n2 := 1;
   lk [6].r1 := 1; lk [6].n1 := 0; lk [6].r2 := 2; lk [6].n2 := 0;
   lk [7].r1 := 2; lk [7].n1 := 1; lk [7].r2 := 1; lk [7].n2 := 1;
   lk [8].r1 := 2; lk [8].n1 := 0; lk [8].r2 := 1; lk [8].n2 := 1;
   fc := nil;
   lg := nil;
   fe := nil; // nil in DEMO VERSION
   ce := nil; // nil in DEMO VERSION
   p1 := nil; // nil in DEMO VERSION
   p2 := nil; // nil in DEMO VERSION
  end;
end;

(* freeing the ressources *)
procedure FreeSyst (sys : TSyst);
begin
 with sys do
  begin
   SetLength (rd, 0);
   SetLength (lk, 0);
  end;
end;

(* display of intermediate results *)
procedure PrintRes (sys : TSyst);
var i, j, k : integer;
begin
 with sys do
  for i := 0 to nb - 1 do
   for j := 0 to 1 do
    begin
     write ('rod #', i + 1, #9, 'end #', j + 1, #9);
     for k := 0 to 2 do write (rd [i].s [j].p [k] : 6 : 3, #9);
     writeln;
    end;
 Writeln ('>>> Potential= ', Potential (@sys) : 15);
 Readln;
end;

(* allow interactivity with user *)
procedure CallBack (sys : pTSyst;
                    opt : plongword); stdcall;
begin
 Inc (Compt);
 Writeln ('iteration # ', Compt);
 PrintRes (sys^);
end;

(* test program *)
begin
 InitSyst (Syst);
 writeln ('start: RETURN to calculate next iteration');
 PrintRes (Syst);
 Res := Solve (pTSyst (@Syst), TCallBack (@CallBack), plongword (@Opts));
 writeln ('result: ', GetStrError (Res), ' ; iteration # ', Compt);
 PrintRes (Syst);
 FreeSyst (Syst);
end.

